import _Object$keys from 'babel-runtime/core-js/object/keys';
import invariant from 'invariant';
import { Store } from '../Store';
import toMap from './toMap';

import { Iterable, OrderedMap, Map } from 'immutable';

import { STORE_INSERT, STORE_REMOVE, STORE_FILTER, STORE_UPDATE } from '../constants/StoreConstants';

export default function createReducerForStore(store) {
  invariant(store && Store.isStore(store), 'Reducer: `store` is expected to be an Orchestra.Store.');

  var identifier = store.identifier;
  var dependencies = store.dependencies;

  var pre = store.getPre();
  var depKeys = _Object$keys(dependencies);
  var Collection = store.getCollection();
  var initial = new Collection(new OrderedMap(), depKeys);

  return function storeReducer() {
    var state = arguments.length <= 0 || arguments[0] === undefined ? initial : arguments[0];
    var action = arguments[1];

    if (identifier !== action.identifier) {
      return state;
    }

    var type = action.type;
    var payload = action.payload;
    var selector = action.selector;

    switch (type) {
      case STORE_INSERT:
        {
          if (Iterable.isKeyed(payload)) {
            var item = pre(payload);
            if (!item) {
              return state;
            }

            var id = item.get('id');
            return state.set(id, item);
          }

          return state.withMutations(function (map) {
            payload.forEach(function (value) {
              var item = pre(value);
              if (!item) {
                return state;
              }

              var id = item.get('id');
              map.set(id, item);
            });
          });
        }

      case STORE_REMOVE:
        {
          if (!payload) {
            return state;
          }

          if (typeof payload === 'string') {
            return state.delete(payload);
          }

          var _id = payload.get('id');
          return state.delete(_id);
        }

      case STORE_FILTER:
        {
          if (typeof selector !== 'function') {
            return state;
          }

          return state.filter(selector);
        }

      case STORE_UPDATE:
        {
          if (typeof selector !== 'function') {
            return state;
          }

          return state.map(function (x) {
            var item = pre(selector(x));
            return item ? item : x;
          });
        }

      default:
        return state;
    }
  };
}