'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.default = createReducerForStore;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _Store = require('../Store');

var _toMap = require('./toMap');

var _toMap2 = _interopRequireDefault(_toMap);

var _immutable = require('immutable');

var _StoreConstants = require('../constants/StoreConstants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createReducerForStore(store) {
  (0, _invariant2.default)(store && _Store.Store.isStore(store), 'Reducer: `store` is expected to be an Orchestra.Store.');

  var identifier = store.identifier;
  var dependencies = store.dependencies;

  var pre = store.getPre();
  var depKeys = (0, _keys2.default)(dependencies);
  var Collection = store.getCollection();
  var initial = new Collection(new _immutable.OrderedMap(), depKeys);

  return function storeReducer() {
    var state = arguments.length <= 0 || arguments[0] === undefined ? initial : arguments[0];
    var action = arguments[1];

    if (identifier !== action.identifier) {
      return state;
    }

    var type = action.type;
    var payload = action.payload;
    var selector = action.selector;

    switch (type) {
      case _StoreConstants.STORE_INSERT:
        {
          if (_immutable.Iterable.isKeyed(payload)) {
            var item = pre(payload);
            if (!item) {
              return state;
            }

            var id = item.get('id');
            return state.set(id, item);
          }

          return state.withMutations(function (map) {
            payload.forEach(function (value) {
              var item = pre(value);
              if (!item) {
                return state;
              }

              var id = item.get('id');
              map.set(id, item);
            });
          });
        }

      case _StoreConstants.STORE_REMOVE:
        {
          if (!payload) {
            return state;
          }

          if (typeof payload === 'string') {
            return state.delete(payload);
          }

          var _id = payload.get('id');
          return state.delete(_id);
        }

      case _StoreConstants.STORE_FILTER:
        {
          if (typeof selector !== 'function') {
            return state;
          }

          return state.filter(selector);
        }

      case _StoreConstants.STORE_UPDATE:
        {
          if (typeof selector !== 'function') {
            return state;
          }

          return state.map(function (x) {
            var item = pre(selector(x));
            return item ? item : x;
          });
        }

      default:
        return state;
    }
  };
}